# epic-sql

A Simple But Powerful MySQL ORM!!!

### Usage

```ts
import {
	SQLSchema,
	SQLBoolean,
	SQLNumber,
	SQLObject,
	SQLString,
	SQLConnector,
	SQLSchemaManager,
	Conditions,
} from "../typescript";

// Create a User Table Schema
const UsersTable = new SQLSchema(
	"users",
	{
		userId: new SQLString({
			isUnique: true, // Unique column
		}),

		fname: new SQLString({
			collation: "utf8_general_ci", // Add special collation type
		}),

		lname: new SQLString({
			isNullable: true, // Data can be null
			defaultValue: "Doe", // Can also set
		}),

		email: new SQLString({
			length: 50, // Can also set the length
		}),

		meta: new SQLObject(), // JSON data is supported

		status: new SQLBoolean(), // Add boolean value

		createdOn: new SQLNumber(), // Date as unix
	},

	["live", "test"] // Add types of Schema
).lock(); // Schema should always be locked before using

// Create a Password Table Schema
const PasswordsTable = new SQLSchema("passwords", {
	passwordId: new SQLString(),

	userId: new SQLString(),

	password: new SQLString(),

	createdOn: new SQLNumber(),
}).lock(); // Schema should always be locked before using

// Working with connection
(async () => {
	try {
		// Create a schema manager
		const connection = await new SQLSchemaManager(
			// Create a connector and pass to manager
			new SQLConnector({
				host: "localhost",
				user: "root",
				database: "test",
				password: "",
			}),

			// Pass all locked schemas to manager as an array
			[UsersTable(), PasswordsTable()]
		).init();

		// Create a client from connection
		const client = await connection.createClient();

		// Pass client to the schema and use schema ORM features
		const NewUser = await UsersTable(client)
			.insertOne(
				{
					userId: 1,
					fname: "John",
					lname: "Doe",
					status: true,
					meta: {},
					createdOn: 123456789,
				},
				"live" // Make sure to select the schema type
			)
			.then(async (insertedUser) => {
				// Add user password
				insertedUser.record.password = (
					await PasswordsTable(client).insertOne({
						passwordId: 1,
						userId: insertedUser.record.userId,
						password: "password123",
						createdOn: 123456789,
					})
				).record;

				// Return user inserted recored with password
				return insertedUser;
			});

		// Log New User
		console.log("New user created::", NewUser);

		// Fetch Users with password relation
		const Users = await UsersTable(client)
			.select(
				[], // You can put columns list to retrive or leave emtpy array as *. You can also add a condition object for extra filtering.
				"live" // Which schema to fetch from.
			)(
				// Returns a function for conditions
				// Call Function with conditions object
				{
					WHERE: {
						userId: 1,
					},
				}
			)
			.then(async (users) => {
				// Lets fetch password for each user
				await Promise.all(
					users.map(async (user, index) => {
						users[index].password = await PasswordsTable(client).selectOne([])(
							new Conditions({
								userId: user.userId,
							})
						);
					})
				);

				// Return users with password
				return users;
			});

		console.log("User fetched with password::", Users);
	} catch (error) {
		console.log("Error:::", error.message || error);
	}
})();
```
